MODULE DragZoom;

(*
 *  This sample Dialog Machine program shows the use of
 *  the dragging mechanism and how to display predefined pictures.
 *)


   FROM DMMaster    IMPORT RunDialogMachine, AddMouseHandler, MouseHandlers;

   FROM DMSystem    IMPORT ScreenWidth, ScreenHeight, MenuBarHeight;

   FROM DMMenus     IMPORT Menu, Command, AccessStatus, Marking,
                           InstallMenu, InstallCommand, InstallQuitCommand;

   FROM DMWindows   IMPORT Window, WindowKind, ScrollBars, WFFixPoint,
                           CloseAttr, ZoomAttr, WindowFrame, RectArea,
                           CreateWindow, AutoRestoreProc;

   FROM DMWindIO    IMPORT DisplayPredefinedPicture, EraseContent, SetPen, LineTo,
                           GetCurMousePos,Drag, SetPos, WriteString, WriteLn;



   CONST
      PictWidth    = 125;   (* values provided by the DMResMover *)
      PictHeight   =  73;
      (* PictID       = 777; (* a grey-scale fish *)           *)
      (* PictID       = 778; (* a black and white sea star *)  *)
      PictID       = 779; (* a black and white fish *)
      PictFileName = "Fish";

   VAR
      firstMenu:   Menu;
      clearComm:   Command;
      resetComm:   Command;

      theWindow:   Window;

      startX, startY: INTEGER; (* start coordinates of the selection rectangle *)



   PROCEDURE Min(a,b: INTEGER): INTEGER;
   BEGIN
      IF a < b THEN
         RETURN a
      ELSE
         RETURN b
      END;
   END Min;


   PROCEDURE DrawRect( MouseX, MouseY: INTEGER);
   BEGIN
      SetPen( startX, startY );
      LineTo( startX, MouseY );
      LineTo( MouseX, MouseY );
      LineTo( MouseX, startY );
      LineTo( startX, startY );
   END DrawRect;


   PROCEDURE DrawZoomedPicture( MouseX, MouseY: INTEGER );
   VAR
      r: RectArea;
   BEGIN
      WITH r DO
         x := Min( startX, MouseX );          (* set (x,y) to lower left corner *)
         y := Min( startY, MouseY);
         w := ABS( startX - MouseX );         (* fit pict size to rectangle *)
         h := ABS( startY - MouseY );
      END;
      DisplayPredefinedPicture( PictFileName, PictID, r);
   END  DrawZoomedPicture;


   PROCEDURE MakeSelection( w: Window );
   BEGIN
      GetCurMousePos( startX, startY );
      Drag( DrawRect, DrawZoomedPicture );
   END MakeSelection;


   PROCEDURE WriteExplanations;
   BEGIN
      SetPos( 2, 1 );
      WriteString( " Define a rectangle by dragging the cursor" ); WriteLn;
      WriteString( " along its diagonal with the mouse button pressed." ); WriteLn;
      WriteLn;
      WriteString( " The picture will be redrawn and scaled to fit the rectangle." );
   END WriteExplanations;


   PROCEDURE ResetPicture;
   VAR
      r: RectArea;
   BEGIN
      WITH r DO
         x := (ScreenWidth() - PictWidth) DIV 2;              (* center picture *)
         y := (ScreenHeight() - MenuBarHeight() - PictHeight) DIV 2;
         w := 0;                                              (* no scaling *)
         h := 0;
      END;
      EraseContent;
      WriteExplanations;
      DisplayPredefinedPicture( PictFileName, PictID, r);
   END  ResetPicture;


   PROCEDURE ClearWindow;
   BEGIN
      EraseContent;
      WriteExplanations;
   END  ClearWindow;


   PROCEDURE JustQuit( VAR ReallyQuit: BOOLEAN );
   BEGIN
      ReallyQuit := TRUE;
   END JustQuit;


   PROCEDURE MakeWindow;
   VAR
      frame: WindowFrame;
   BEGIN
      WITH frame DO
         x :=  0; y :=  0;
         w := ScreenWidth(); h := ScreenHeight() - MenuBarHeight();
      END;
      CreateWindow( theWindow, FixedLocation,
                    WithoutScrollBars, WithoutCloseBox, WithoutZoomBox,
                    bottomLeft, frame, "Drag & Zoom", AutoRestoreProc );
   END MakeWindow;


   PROCEDURE MakeMenuBar;
   BEGIN
      InstallMenu( firstMenu, "Drag&Zoom", enabled );
      InstallCommand( firstMenu, clearComm, "Clear window contents",
                      ClearWindow, enabled, unchecked );
      InstallCommand( firstMenu, resetComm, "Redraw picture in original size",
                      ResetPicture, enabled, unchecked );
      InstallQuitCommand( "Quit", JustQuit, "Q" );
   END MakeMenuBar;



BEGIN
   MakeMenuBar;
   MakeWindow;
   ResetPicture;
   AddMouseHandler( WindowContent, MakeSelection, 1);
   RunDialogMachine;
END DragZoom.