MODULE FontStyleTest;

(*
 *  This sample Dialog Machine program shows the use of a dialog
 *  box to change font characteristics.
 *
 *  Exercise:
 *     Watch out for the font name (Geneva, NewYork...) and why it is *NOT*
 *     kept in the dialog box:  whenever you select "Change text attributes"
 *     the font name is back to Chicago.  Try to make it "remember" the font
 *     name you had selected last.
 *)

   FROM DMMaster     IMPORT RunDialogMachine;

   FROM DMMenus      IMPORT Menu, Command, AccessStatus, Marking,
                            InstallMenu, InstallCommand, InstallQuitCommand,
                            InstallAliasChar;

   FROM DMWindows    IMPORT Window, WindowKind, ScrollBars, WFFixPoint,
                            CloseAttr, ZoomAttr, WindowFrame, CreateWindow,
                            AutoRestoreProc;

   FROM DMEntryForms IMPORT FormFrame, DefltUse,
                            WriteLabel, IntField, CheckBox, UseEntryForm,
                            RadioButtonID, RadioButton, DefineRadioButtonSet;

   FROM DMWindIO     IMPORT EraseContent, SetPen, WriteString, WriteInt,
                            WindowFont, FontStyles, FontStyle, SetWindowFont,
                            CellWidth, CellHeight;



   CONST
      TheText = "Time has come, the walrus said..";

   VAR
      firstMenu:        Menu; 
      writeComm:        Command;
      changeComm:       Command;

      theWindow:        Window;

      charHeight:       INTEGER;
      isBold, isItalic,
      isUnderlined:     BOOLEAN;



   PROCEDURE WriteText;
   BEGIN
      EraseContent;
      SetPen( 20, 120 ); WriteString( TheText );
      SetPen( 10,  50 ); WriteString( "CellWidth  CellHeight" ); 
      SetPen( 10,  30 ); WriteInt( CellWidth(), 7 ); WriteInt( CellHeight(), 11 );
   END WriteText;


   PROCEDURE SetTextAttributes;
   VAR
      ff:          FormFrame;
      ok:          BOOLEAN;
      fontID:      WindowFont;
      fontButtons: RadioButtonID;
      style:       FontStyle;
      genevaRB:    RadioButtonID;
      monacoRB:    RadioButtonID;
      chicagoRB:   RadioButtonID;
      newYorkRB:   RadioButtonID;

   BEGIN
      IntField(   2, 2, 4, charHeight, useAsDeflt, 0, 30000 );
      WriteLabel( 2, 7, "Font size" );

      DefineRadioButtonSet( fontButtons );
      RadioButton( genevaRB,  4, 2, "Geneva" );
      RadioButton( monacoRB,  5, 2, "Monaco" );
      RadioButton( chicagoRB, 6, 2, "Chicago" );
      RadioButton( newYorkRB, 7, 2, "NewYork" );
      fontButtons := chicagoRB;

      CheckBox( 4, 20, "bold",       isBold );
      CheckBox( 5, 20, "italic",     isItalic );
      CheckBox( 6, 20, "underlined", isUnderlined );

      ff.x := 40; ff.y := 40; ff.lines := 9; ff.columns := 40;
      UseEntryForm( ff, ok );
      IF ok THEN
         style := FontStyle{};
         IF isBold THEN
            INCL( style, bold );
         END;
         IF isItalic THEN
            style := style + FontStyle{ italic };           (* alternative way *)
         END;
         IF isUnderlined THEN
            INCL( style, underline );
         END;
         IF fontButtons = genevaRB THEN
            fontID := Geneva;
         ELSIF fontButtons = monacoRB THEN
            fontID := Monaco;
         ELSIF fontButtons = chicagoRB THEN
            fontID := Chicago;
         ELSIF fontButtons = newYorkRB THEN
            fontID := NewYork;
         END;
         SetWindowFont( fontID, charHeight, style );
      END;
      WriteText;
   END SetTextAttributes;


   PROCEDURE JustQuit( VAR reallyQuit: BOOLEAN );
   BEGIN
      reallyQuit := TRUE;
   END JustQuit;


   PROCEDURE MakeWindow;
   VAR
      frame: WindowFrame;
   BEGIN
      WITH frame DO
         x :=  50; y :=  50;
         w := 450; h := 250;
      END;
      CreateWindow( theWindow, FixedLocation,
                    WithoutScrollBars, WithoutCloseBox, WithoutZoomBox,
                    bottomLeft, frame, "Font Test", AutoRestoreProc );
   END MakeWindow;


   PROCEDURE MakeMenuBar;
   BEGIN
      InstallMenu( firstMenu, "Fonts&Styles", enabled );
      InstallCommand( firstMenu, writeComm, "Write text",
                      WriteText, enabled, unchecked );
      InstallCommand( firstMenu, changeComm, "Change text attributes ...",
                      SetTextAttributes, enabled, unchecked );
      InstallQuitCommand( "Quit", JustQuit, "Q" );

      InstallAliasChar( firstMenu, writeComm,  "W");
      InstallAliasChar( firstMenu, changeComm, "A");
   END MakeMenuBar;



BEGIN
   charHeight := 12;
   isBold := FALSE; isItalic := FALSE; isUnderlined := FALSE;

   MakeMenuBar;
   MakeWindow;
   WriteText;
   RunDialogMachine;
END FontStyleTest.