MODULE MouseTest;

(*
 *  Demonstrates and tests some basic mouse functions of the
 *  Dialog Machine.
 *)

   FROM DMMaster   IMPORT RunDialogMachine, DialogMachineTask;
   FROM DMMenus    IMPORT Menu, Command, InstallMenu, InstallCommand,
                          AccessStatus, Marking, InstallAliasChar,
                          InstallQuitCommand;
   FROM DMWindows  IMPORT Window, WindowKind, ScrollBars, WFFixPoint,
                          CloseAttr, ZoomAttr, WindowFrame, RectArea,
                          CreateWindow, AutoRestoreProc;
   FROM DMWindIO   IMPORT GetCurMousePos, RectClicked, SetPos,
                          WriteString, WriteInt, GreyContent, pat, Area,
                          SetMode, GetMode, PaintMode;



   VAR
      firstMenu:  Menu; 
      startComm:  Command;
      stopComm:   Command;
      
      theWindow:  Window;
      greyBox:    RectArea;

      monitoringIsOn: BOOLEAN;



   PROCEDURE MonitorMouse;
   VAR
      mouseX, mouseY: INTEGER;
      savedMouseX, savedMouseY: INTEGER;
      counter:        INTEGER;
      savedMode:      PaintMode;
   BEGIN
      savedMouseX := MIN(INTEGER);
      savedMouseY := MIN(INTEGER);
      monitoringIsOn := TRUE;
      counter := 0;
      WHILE monitoringIsOn DO
         GetCurMousePos( mouseX, mouseY );
         IF (savedMouseX#mouseX) OR (savedMouseY#mouseY) THEN
            SetPos( 3, 3 ); WriteString( "                           " );
            SetPos( 3, 3 ); WriteInt( mouseX, 5 ); WriteInt( mouseY, 5 );
            savedMouseX := mouseX;  savedMouseY := mouseY;
         END;
         IF RectClicked( greyBox ) THEN
            SetPos( 5, 3 ); WriteString( "Mouse click" );
            counter := 50;
         ELSE
            IF counter > 0 THEN
               DEC( counter );
            ELSIF counter = 0 THEN
               GetMode( savedMode );
               SetMode( erase );
               SetPos( 5, 3 ); WriteString( "Mouse click" );
               SetMode( savedMode );
            END;
         END;
         DialogMachineTask;  (* allows user actions i.e. menu
                                selection during monitoring   *)
      END;
   END MonitorMouse;


   PROCEDURE StopMonitoring;
   BEGIN
      monitoringIsOn := FALSE;
   END StopMonitoring;


   PROCEDURE JustQuit( VAR reallyQuit: BOOLEAN );
   BEGIN
      StopMonitoring;
      reallyQuit := TRUE;
   END JustQuit;


   PROCEDURE MakeMenuBar;
   BEGIN
      InstallMenu( firstMenu, "Mouse", enabled );
      InstallCommand( firstMenu, startComm, "Start monitoring mouse",
                      MonitorMouse, enabled, unchecked );
      InstallCommand( firstMenu, stopComm, "Stop monitoring mouse",
                      StopMonitoring, enabled, unchecked );
      InstallQuitCommand( "Quit", JustQuit,   "Q" );

      InstallAliasChar( firstMenu, startComm, "M" );
      InstallAliasChar( firstMenu, stopComm,  "E" );
   END MakeMenuBar;


   PROCEDURE MakeWindow;
   VAR
      frame: WindowFrame;
   BEGIN
      WITH frame DO
         x :=  50; y :=  45;
         w := 280; h := 180;
      END;
      CreateWindow( theWindow, FixedSize,
                    WithoutScrollBars, WithoutCloseBox, WithoutZoomBox,
                    bottomLeft, frame, "Test", AutoRestoreProc );

      SetPos( 2, 3 ); WriteString( "Mouse position (X,Y)" );

      WITH greyBox DO
         x := 50; y := 45;
         w := 50; h := 50;
      END;
      Area( greyBox, pat[ lightGrey ] );
   END MakeWindow;



BEGIN

   monitoringIsOn := FALSE;

   MakeMenuBar;
   MakeWindow;
   RunDialogMachine;
END MouseTest.