MODULE WindowTest;

(*
 *  This program shows the use of several overlapping windows and their handling 
 *  with the mouse. It is also a test for the AutoRestore mechanism.
 *)

   FROM DMMaster   IMPORT RunDialogMachine;

   FROM DMMenus    IMPORT Menu, Command, InstallMenu, InstallCommand,
                          Separator, InstallSeparator, AccessStatus, Marking,
                          InstallQuitCommand, InstallAliasChar;

   FROM DMWindows  IMPORT Window, CreateWindow, WindowKind, ScrollBars,
                          CloseAttr, ZoomAttr, WindowFrame, WFFixPoint,
                          RectArea, RemoveWindow, RemoveAllWindows,
                          FrontWindow, AutoRestoreProc, AddWindowHandler,
                          WindowHandlers;

   FROM DMWindIO   IMPORT Dot, LineTo, SetPen, GreyContent, pat, Area, SetPos,
                          WriteString, SelectForOutput;



   VAR
      firstMenu:            Menu;
      openWindowComm:       Command;
      drawComm:             Command;
      closeAllWindowsComm:  Command;
      closeFrontWindowComm: Command;

      windowTable:          ARRAY[ 1..20 ] OF Window;
      nrOfWindows:          INTEGER;



   PROCEDURE JustQuit( VAR reallyQuit: BOOLEAN );
   BEGIN
      reallyQuit := TRUE;
   END JustQuit;


   PROCEDURE DecNrOfWindows( w: Window );
   BEGIN
      DEC( nrOfWindows );
   END DecNrOfWindows;


   PROCEDURE Draw;
   VAR
      greyBox: RectArea;
   BEGIN
      SelectForOutput( FrontWindow() );
      Dot( 90, 20 );
      LineTo( 110,  20 );
      LineTo( 105,  40 );
      LineTo( 160,  30 );
      LineTo( 120,  70 );
      LineTo( 142,  62 );
      LineTo( 110, 100 );
      LineTo( 130,  94 );
      LineTo( 100, 140 );
      LineTo(  70,  94 );
      LineTo(  90, 100 );
      LineTo(  58,  62 );
      LineTo(  80,  70 );
      LineTo(  40,  30 );
      LineTo(  95,  40 );
      LineTo(  90,  20 );

      greyBox.x := 20; greyBox.y := 160; greyBox.w := 160; greyBox.h := 30;
      Area( greyBox, pat[ lightGrey ] );
      SetPos( 2, 4 ); WriteString( "happy New Year!" );
   END Draw;


   PROCEDURE MakeWindow;
   VAR
      frame: WindowFrame;
   BEGIN
      IF nrOfWindows<20 THEN
         WITH frame DO
            x :=  20 + nrOfWindows*10;
            y :=  20 + nrOfWindows*10;
            w := 200;
            h := 200;
         END;

         INC( nrOfWindows );
         CreateWindow( windowTable[ nrOfWindows ], GrowOrShrinkOrDrag,
                       WithBothScrollBars, WithCloseBox, WithZoomBox,
                       bottomLeft, frame, "Merry Christmas!", AutoRestoreProc );
         AddWindowHandler( windowTable [ nrOfWindows ], closing, DecNrOfWindows, 0 );
      END;
   END MakeWindow;


   PROCEDURE DeleteFrontWindow;
   VAR
      w: Window;
   BEGIN
      w := FrontWindow();
      RemoveWindow( w );
   END DeleteFrontWindow;


   PROCEDURE DeleteAllWindows;
   BEGIN
      RemoveAllWindows;
      nrOfWindows := 0;
   END DeleteAllWindows;


   PROCEDURE MakeMenuBar;
   BEGIN
      InstallMenu( firstMenu, "Windows", enabled );
      InstallCommand( firstMenu, openWindowComm, "Open new window",
                      MakeWindow, enabled, unchecked );
      InstallSeparator( firstMenu, line );
      InstallCommand( firstMenu, drawComm, "Draw into FrontWindow",
                      Draw, enabled, unchecked );
      InstallCommand( firstMenu, closeFrontWindowComm, "Close FrontWindow",
                      DeleteFrontWindow, enabled, unchecked );
      InstallCommand( firstMenu, closeAllWindowsComm, "Close all windows",
                      DeleteAllWindows, enabled, unchecked );
      InstallQuitCommand( "Quit", JustQuit, "Q" );

      InstallAliasChar( firstMenu, openWindowComm,       "N" );
      InstallAliasChar( firstMenu, drawComm,             "D" );
      InstallAliasChar( firstMenu, closeFrontWindowComm, "W" );
      InstallAliasChar( firstMenu, closeAllWindowsComm,  "A" );
   END MakeMenuBar;



BEGIN
   nrOfWindows := 0;
   MakeMenuBar;
   RunDialogMachine;
END WindowTest.